@@ -215,6 +215,9 @@ namespace Arc {
     /// Set format for this log destination.
     void setFormat(const LogFormat& newformat);
 
+    /// Set format for any new log destination.
+    static void setDefaultFormat(const LogFormat& newformat);
+
     /// Set a prefix for this log destination to be logged before messages.
     /** The prefix is prepended to messages in all formats except EmptyFormat.
      * \since Added in 4.0.0.
@@ -527,7 +530,10 @@ namespace Arc {
     void removeDestinations(void);
 
     /// Remove all LogDestinations and delete LogDestination objects.
-    void deleteDestinations(void);
+    /** A LogDestination to not delete can be specified in the exclude parameter.
+     * @param exclude Do not delete this destination
+     */
+    void deleteDestinations(LogDestination* exclude=NULL);
 
     /// Sets the logging threshold.
     /** This method sets the threshold of the Logger. Any message sent
