@@ -126,11 +126,10 @@ namespace Arc {
 
       // TODO: stdin, stdout, stderr redirections (Apache/BSD license)
 
-      char **args = const_cast<char**>(argv_.data());
+      std::list<std::string>::const_iterator argp = argv_.begin();
       std::string cmd = "";
-      for (int i = 0; args[i] != NULL; i++) {
-        std::string a(args[i]);
-        cmd += (a + " ");
+      for (; argp != argv_.end(); ++argp) {
+        cmd += ((*argp) + " ");
       }
       int result = CreateProcess(NULL,
                                  (LPSTR)cmd.c_str(),
