@@ -1,8 +1,15 @@
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <cppunit/extensions/HelperMacros.h>
 
 #include <utime.h>
 #include <sys/stat.h>
+#ifndef WIN32
 #include <sys/utsname.h>
+#endif
+#include <unistd.h>
 #ifdef WIN32
 #include <Winsock2.h> // for gethostname()
 #endif
@@ -67,13 +74,16 @@ void FileLockTest::TestFileLockAcquire() {
 
   // construct hostname
   char hostname[256];
-  CPPUNIT_ASSERT_EQUAL_MESSAGE("Cannot determine hostname from gethostname()", 0, gethostname(hostname, sizeof(hostname)));
+  if(gethostname(hostname, sizeof(hostname)) != 0) hostname[0] = '\0';
   std::string host(hostname);
   CPPUNIT_ASSERT_EQUAL(Arc::tostring(getpid()) + "@" + host, lock_pid);
 
-  // set old modification time
+  bool lock_removed = false;
   struct utimbuf times;
   time_t t = 1;
+#ifndef WIN32
+  // set old modification time
+  // utime does not work on windows - skip for windows
   times.actime = t;
   times.modtime = t;
   CPPUNIT_ASSERT_EQUAL(0, utime(lock_file.c_str(), &times));
@@ -81,9 +91,9 @@ void FileLockTest::TestFileLockAcquire() {
   CPPUNIT_ASSERT_EQUAL(t, fileStat.st_mtime);
 
   // call acquire() again - should succeed and make new lock file
-  bool lock_removed = false;
   CPPUNIT_ASSERT(lock.acquire(lock_removed));
   CPPUNIT_ASSERT(lock_removed);
+#endif
 
   // look at modification time - should not be more than 1 second old
   CPPUNIT_ASSERT_EQUAL_MESSAGE("Could not stat lock file " + lock_file, 0, stat(lock_file.c_str(), &fileStat));
@@ -103,11 +113,20 @@ void FileLockTest::TestFileLockAcquire() {
   CPPUNIT_ASSERT(!lock.acquire(lock_removed));
   CPPUNIT_ASSERT(!lock_removed);
 
+#ifndef WIN32
   // try again with a non-existent pid
+  // windows can't check for running pid - skip for windows
   _createFile(lock_file, "99999@" + host);
   lock_removed = false;
   CPPUNIT_ASSERT(lock.acquire(lock_removed));
   CPPUNIT_ASSERT(lock_removed);
+#endif
+
+  // badly formatted pid
+  _createFile(lock_file, "abcd@" + host);
+  lock_removed = false;
+  CPPUNIT_ASSERT(!lock.acquire(lock_removed));
+  CPPUNIT_ASSERT(!lock_removed);
 
   // set small timeout
   CPPUNIT_ASSERT_EQUAL(0, remove(lock_file.c_str()));
@@ -115,7 +134,9 @@ void FileLockTest::TestFileLockAcquire() {
   lock = Arc::FileLock(filename, 1);
   CPPUNIT_ASSERT(lock.acquire(lock_removed));
   CPPUNIT_ASSERT(!lock_removed);
-  sleep(2);
+  // use longer sleep because times and sleeps are very
+  // approximate on windows
+  sleep(4);
   CPPUNIT_ASSERT(lock.acquire(lock_removed));
   CPPUNIT_ASSERT(lock_removed);
 
@@ -136,7 +157,9 @@ void FileLockTest::TestFileLockAcquire() {
   lock = Arc::FileLock(filename);
   CPPUNIT_ASSERT(!lock.acquire());
 
+#ifndef WIN32
   // set old modification time - acquire should now succeed
+  // utime does not work on windows - skip for windows
   times.actime = t;
   times.modtime = t;
   CPPUNIT_ASSERT_EQUAL(0, utime(lock_file.c_str(), &times));
@@ -144,13 +167,17 @@ void FileLockTest::TestFileLockAcquire() {
   CPPUNIT_ASSERT_EQUAL(t, fileStat.st_mtime);
 
   CPPUNIT_ASSERT(lock.acquire());
+#endif
 
+#ifndef WIN32
   // create lock with empty hostname - acquire should still work
+  // windows can't check for running pid - skip for windows
   lock = Arc::FileLock(filename);
   _createFile(lock_file, "99999@");
   lock_removed = false;
   CPPUNIT_ASSERT(lock.acquire(lock_removed));
   CPPUNIT_ASSERT(lock_removed);
+#endif
 }
 
 void FileLockTest::TestFileLockRelease() {
@@ -166,7 +193,7 @@ void FileLockTest::TestFileLockRelease() {
 
   // construct hostname
   char hostname[256];
-  CPPUNIT_ASSERT_EQUAL_MESSAGE("Cannot determine hostname from gethostname()", 0, gethostname(hostname, sizeof(hostname)));
+  if(gethostname(hostname, sizeof(hostname)) != 0) hostname[0] = '\0';
   std::string host(hostname);
 
   // create a valid lock file with this pid
@@ -220,41 +247,41 @@ void FileLockTest::TestFileLockCheck() {
   struct stat fileStat;
   CPPUNIT_ASSERT(stat(lock_file.c_str(), &fileStat) != 0);
   Arc::FileLock lock(filename);
-  CPPUNIT_ASSERT(!lock.check());
+  CPPUNIT_ASSERT_EQUAL(-1, lock.check());
 
   // construct hostname
   char hostname[256];
-  CPPUNIT_ASSERT_EQUAL_MESSAGE("Cannot determine hostname from gethostname()", 0, gethostname(hostname, sizeof(hostname)));
+  if(gethostname(hostname, sizeof(hostname)) != 0) hostname[0] = '\0';
   std::string host(hostname);
 
   // create a valid lock file with this pid
   _createFile(lock_file, std::string(Arc::tostring(getpid()) + "@" + host));
 
   lock = Arc::FileLock(filename);
-  CPPUNIT_ASSERT(lock.check());
+  CPPUNIT_ASSERT_EQUAL(0, lock.check());
 
   // create a lock with a different pid
   _createFile(lock_file, std::string("1@" + host));
-  CPPUNIT_ASSERT(!lock.check());
+  CPPUNIT_ASSERT_EQUAL(1, lock.check());
 
   // create lock with different host
   CPPUNIT_ASSERT_EQUAL(0, remove(lock_file.c_str()));
   _createFile(lock_file, std::string(Arc::tostring(getpid()) + "@mybadhost.org"));
-  CPPUNIT_ASSERT(!lock.check());
+  CPPUNIT_ASSERT_EQUAL(-1, lock.check());
 
   // create an empty lock file - check should fail
   _createFile(lock_file, "");
   lock = Arc::FileLock(filename);
-  CPPUNIT_ASSERT(!lock.check());
+  CPPUNIT_ASSERT_EQUAL(-1, lock.check());
 
   // set use_pid false, check should succeed now
   lock = Arc::FileLock(filename, 30, false);
-  CPPUNIT_ASSERT(lock.check());
+  CPPUNIT_ASSERT_EQUAL(0, lock.check());
 
   // create lock with empty hostname - check should still be ok
   lock = Arc::FileLock(filename);
   _createFile(lock_file, std::string(Arc::tostring(getpid()) + "@"));
-  CPPUNIT_ASSERT(lock.check());
+  CPPUNIT_ASSERT_EQUAL(0, lock.check());
 }
 
 bool FileLockTest::_createFile(const std::string& filename, const std::string& text) {
