@@ -33,7 +33,7 @@ static Arc::Plugin* get_mcc_client(Arc::PluginArgument* arg) {
     return new ArcMCCHTTP::MCC_HTTP_Client((Arc::Config*)(*mccarg),mccarg);
 }
 
-Arc::PluginDescriptor ARC_PLUGINS_TABLE_NAME[] = {
+extern Arc::PluginDescriptor const ARC_PLUGINS_TABLE_NAME[] = {
     { "http.service", "HED:MCC", NULL, 0, &get_mcc_service },
     { "http.client",  "HED:MCC", NULL, 0, &get_mcc_client  },
     { NULL, NULL, NULL, 0, NULL }
@@ -184,6 +184,7 @@ static MCC_Status make_raw_fault(Message& outmsg,const MCC_Status& desc) {
   PayloadRaw* outpayload = new PayloadRaw;
   std::string errstr = (std::string)desc;
   if(!errstr.empty()) outpayload->Insert(errstr.c_str(),0);
+  outmsg.Payload(outpayload);
   return desc;
 }
 
@@ -554,6 +555,7 @@ MCC_Status MCC_HTTP_Client::process(Message& inmsg,Message& outmsg) {
   outmsg.Payload(outpayload);
   outmsg.Attributes()->set("HTTP:CODE",tostring(outpayload->Code()));
   outmsg.Attributes()->set("HTTP:REASON",outpayload->Reason());
+  outmsg.Attributes()->set("HTTP:KEEPALIVE",outpayload->KeepAlive()?"TRUE":"FALSE");
   for(std::map<std::string,std::string>::const_iterator i =
       outpayload->Attributes().begin();i!=outpayload->Attributes().end();++i) {
     outmsg.Attributes()->add("HTTP:"+i->first,i->second);
