@@ -13,7 +13,7 @@
 
 namespace ArcSHCLegacy {
 
-int AuthUser::match_lcas(const char* line) {
+AuthResult AuthUser::match_lcas(const char* line) {
   // TODO: escape
   // TODO: hardcoded 60s timeout
   std::string lcas_plugin = "60 \""+
@@ -22,7 +22,7 @@ int AuthUser::match_lcas(const char* line) {
   lcas_plugin+=std::string("\"")+DN()+"\" ";
   lcas_plugin+=std::string("\"")+proxy()+"\" ";
   lcas_plugin+=line;
-  int res = match_plugin(lcas_plugin.c_str());
+  AuthResult res = match_plugin(lcas_plugin.c_str());
   return res;
 }
 
