@@ -126,6 +126,10 @@ namespace DataStaging {
       args.push_back("--topt");
       args.push_back("maxinacttime="+Arc::tostring(transfer_params.max_inactivity_time));
 
+      if (dtr->get_source()->CheckSize()) {
+        args.push_back("--size");
+        args.push_back(Arc::tostring(dtr->get_source()->GetSize()));
+      }
       if (dtr->get_source()->CheckCheckSum()) {
         std::string csum(dtr->get_source()->GetCheckSum());
         std::string::size_type pos(csum.find(':'));
@@ -241,8 +245,15 @@ namespace DataStaging {
     }
   }
 
-  bool DataDeliveryLocalComm::CheckComm(DTR_ptr dtr, std::vector<std::string>& allowed_dirs) {
+  bool DataDeliveryLocalComm::CheckComm(DTR_ptr dtr, std::vector<std::string>& allowed_dirs, std::string& load_avg) {
     allowed_dirs.push_back("/");
+    double avg[3];
+    if (getloadavg(avg, 3) != 3) {
+      load_avg = "-1";
+    } else {
+      load_avg = Arc::tostring(avg[1]);
+    }
+
     return true;
   }
 
