@@ -14,7 +14,6 @@
 #include <arc/Utils.h>
 #include <arc/XMLNode.h>
 
-#include "ConfigUtils.h"
 #include "CoreConfig.h"
 #include "../run/RunParallel.h"
 
@@ -35,6 +34,7 @@ namespace ARex {
 
 Arc::Logger GMConfig::logger(Arc::Logger::getRootLogger(), "GMConfig");
 static std::string empty_string("");
+static std::list<std::string> empty_string_list;
 
 GMConfig::GMConfig(const std::string& conf): conffile(conf) {
   SetDefaults();
@@ -67,6 +67,7 @@ GMConfig::GMConfig(const Arc::XMLNode& node): xml_cfg(node) {
 void GMConfig::SetDefaults() {
   conffile_is_temp = false;
   job_log = NULL;
+  jobs_metrics = NULL;
   job_perf_log = NULL;
   cont_plugins = NULL;
   cred_plugin = NULL;
@@ -84,6 +85,7 @@ void GMConfig::SetDefaults() {
   max_jobs_total = -1;
   max_jobs = -1;
   max_jobs_per_dn = -1;
+  max_scripts = -1;
 
   deleg_db = deleg_db_bdb;
 
@@ -272,6 +274,16 @@ GMConfig::deleg_db_t GMConfig::DelegationDBType() const {
   return deleg_db;
 }
 
+const std::string & GMConfig::ForcedVOMS(const char * queue) const {
+  std::map<std::string,std::string>::const_iterator pos = forced_voms.find(queue);
+  return (pos == forced_voms.end()) ? empty_string : pos->second;
+}
+
+const std::list<std::string> & GMConfig::AuthorizedVOs(const char * queue) const {
+  std::map<std::string, std::list<std::string> >::const_iterator pos = authorized_vos.find(queue);
+  return (pos == authorized_vos.end()) ? empty_string_list : pos->second;
+}
+
 bool GMConfig::Substitute(std::string& param, const Arc::User& user) const {
   std::string::size_type curpos = 0;
   for (;;) {
@@ -406,6 +418,7 @@ bool GMConfig::ExternalHelper::run(const GMConfig& config) {
   proc->AssignInitializer(&ExternalHelperInitializer, const_cast<char*>(config.HelperLog().c_str()));
   if (proc->Start()) return true;
   delete proc;
+  proc = NULL;
   logger.msg(Arc::ERROR, "Helper process start failed: %s", command);
   // start failed, doing nothing - maybe in the future
   return false;
