@@ -26,15 +26,6 @@ enum job_state_t {
   JOB_STATE_UNDEFINED  = 8
 };
 
-struct job_state_rec_t {
-  job_state_t id;
-  const char* name;
-  char mail_flag;
-};
-
-/// Maps job state to state name and flag for email at that state
-extern job_state_rec_t states_all[JOB_STATE_UNDEFINED+1];
-
 /// Number of job states
 #define JOB_STATE_NUM (JOB_STATE_UNDEFINED+1)
 
@@ -65,14 +56,17 @@ class GMJob {
   JobLocalDescription* local;
   // Job's owner
   Arc::User user;
-  // Reties left (should be reset for each state change)
-  int retries;
-  // Time to perform the next retry
-  time_t next_retry;
   // Used to determine data transfer share (eg DN, VOMS VO)
   std::string transfer_share;
   // Start time of job i.e. when it first moves to PREPARING
   time_t start_time;
+
+  struct job_state_rec_t {
+    const char* name;
+    char mail_flag;
+  };
+  /// Maps job state to state name and flag for email at that state
+  static job_state_rec_t const states_all[JOB_STATE_NUM];
  public:
   // external utility being run to perform tasks like stage-in/out,
   //   submit/cancel. (todo - move to private)
@@ -86,17 +80,23 @@ class GMJob {
   GMJob(const JobId &job_id,const Arc::User& user,const std::string &dir = "",job_state_t state = JOB_STATE_UNDEFINED);
   GMJob(void);
   GMJob(const GMJob &job);
+  GMJob& operator=(const GMJob &job);
   ~GMJob(void);
   job_state_t get_state() const { return job_state; };
   const char* get_state_name() const;
+  char get_state_mail_flag() const;
   static const char* get_state_name(job_state_t st);
+  static char get_state_mail_flag(job_state_t st);
   static job_state_t get_state(const char* state);
   const JobId& get_id() const { return job_id; };
   std::string SessionDir(void) const { return session_dir; };
   void AddFailure(const std::string &reason) { failure_reason+=reason; failure_reason+="\n"; };
+  /// Retrieve current failure reason (both in memory and stored in control dir).
+  /// For non-failed jobs returned string is empty.
   std::string GetFailure(const GMConfig& config) const;
-  JobLocalDescription* get_local(void) const { return local; };
-  void set_local(JobLocalDescription* desc) { local=desc; };
+  /// Check if job is marked as failed (slightly faster than GetFailure).
+  /// For failed job returns true, non-failed - false.
+  bool CheckFailure(const GMConfig& config) const;
   bool operator==(const GMJob& job) const { return (job_id == job.job_id); };
   bool operator==(const JobId &id) const { return (job_id == id); };
   bool operator!=(const JobId &id) const { return (job_id != id); };
@@ -104,7 +104,9 @@ class GMJob {
   const Arc::User& get_user() const { return user;}
   void set_share(std::string share);
   // Force 'local' to be created and read from file if not already available
-  bool GetLocalDescription(const GMConfig& config);
+  JobLocalDescription* GetLocalDescription(const GMConfig& config);
+  // Use only preloaded local
+  JobLocalDescription* GetLocalDescription() const;
   void Start() { start_time = time(NULL); };
   time_t GetStartTime() const { return start_time; };
   void PrepareToDestroy(void);
