@@ -13,6 +13,7 @@
 #include "../run/RunParallel.h"
 #include "../mail/send_mail.h"
 #include "../log/JobLog.h"
+#include "../log/JobsMetrics.h"
 #include "../misc/proxy.h"
 #include "../../delegation/DelegationStores.h"
 #include "../../delegation/DelegationStore.h"
@@ -38,6 +39,7 @@ static Arc::Logger& logger = Arc::Logger::getRootLogger();
 JobsList::JobsList(const GMConfig& gmconfig) :
     config(gmconfig), staging_config(gmconfig), old_dir(NULL), dtr_generator(NULL), job_desc_handler(config), jobs_pending(0) {
   for(int n = 0;n<JOB_STATE_NUM;n++) jobs_num[n]=0;
+  jobs_scripts = 0;
   jobs.clear();
 }
  
@@ -58,7 +60,7 @@ void JobsList::UpdateJobCredentials(JobsList::iterator &i) {
   if(GetLocalDescription(i)) {
     std::string delegation_id = i->local->delegationid;
     if(!delegation_id.empty()) {
-      ARex::DelegationStores* delegs = config.delegations;
+      ARex::DelegationStores* delegs = config.GetDelegations();
       if(delegs) {
         std::string cred;
         if((*delegs)[config.DelegationDir()].GetCred(delegation_id,i->local->DN,cred)) {
@@ -71,6 +73,8 @@ void JobsList::UpdateJobCredentials(JobsList::iterator &i) {
 
 void JobsList::SetJobState(JobsList::iterator &i, job_state_t new_state, const char* reason) {
   if(i->job_state != new_state) {
+    JobsMetrics* metrics = config.GetJobsMetrics();
+    if(metrics) metrics->ReportJobStateChange(i->job_id, new_state, i->job_state);
     std::string msg = Arc::Time().str(Arc::UTCTime);
     msg += " Job state change "; 
     msg += i->get_state_name();
@@ -92,8 +96,8 @@ void JobsList::SetJobState(JobsList::iterator &i, job_state_t new_state, const c
 
 bool JobsList::AddJobNoCheck(const JobId &id,JobsList::iterator &i,uid_t uid,gid_t gid){
   i=jobs.insert(jobs.end(),GMJob(id, Arc::User(uid)));
-  i->keep_finished=config.keep_finished;
-  i->keep_deleted=config.keep_deleted;
+  i->keep_finished=config.KeepFinished();
+  i->keep_deleted=config.KeepDeleted();
   if (!GetLocalDescription(i)) {
     // safest thing to do is add failure and move to FINISHED
     i->AddFailure("Internal error");
@@ -235,7 +239,7 @@ bool JobsList::FailedJob(const JobsList::iterator &i,bool cancel) {
     r = false;
   }
   // Convert delegation ids to credential paths.
-  std::string default_cred = config.control_dir + "/job." + i->get_id() + ".proxy";
+  std::string default_cred = config.ControlDir() + "/job." + i->get_id() + ".proxy";
   for(std::list<FileData>::iterator f = job_desc.outputdata.begin();
                                    f != job_desc.outputdata.end(); ++f) {
     if(f->has_lfn()) {
@@ -243,7 +247,7 @@ bool JobsList::FailedJob(const JobsList::iterator &i,bool cancel) {
         f->cred = default_cred;
       } else {
         std::string path;
-        ARex::DelegationStores* delegs = config.delegations;
+        ARex::DelegationStores* delegs = config.GetDelegations();
         if(delegs && i->local) path = (*delegs)[config.DelegationDir()].FindCred(f->cred,i->local->DN);
         f->cred = path;
       }
@@ -280,9 +284,20 @@ bool JobsList::GetLocalDescription(const JobsList::iterator &i) {
   return true;
 }
 
+void JobsList::CleanChildProcess(const JobsList::iterator i) {
+  delete i->child; i->child=NULL;
+  if((i->job_state == JOB_STATE_SUBMITTING) || (i->job_state == JOB_STATE_CANCELING)) --jobs_scripts;
+}
+
 bool JobsList::state_submitting(const JobsList::iterator &i,bool &state_changed,bool cancel) {
   if(i->child == NULL) {
     // no child was running yet, or recovering from fault 
+    if((config.MaxScripts()!=-1) && (jobs_scripts>=config.MaxScripts())) {
+      //logger.msg(Arc::WARNING,"%s: Too many LRMS scripts running - limit is %u",
+      //                     i->job_id,config.MaxScripts());
+      // returning true but not advancing to next state should cause retry
+      return true;
+    }
     // write grami file for submit-X-job
     // TODO: read existing grami file to check if job is already submitted
     if(!(i->GetLocalDescription(config))) {
@@ -323,8 +338,8 @@ bool JobsList::state_submitting(const JobsList::iterator &i,bool &state_changed,
         return true;
       }
     }
-    std::string grami = config.control_dir+"/job."+(*i).job_id+".grami";
-    cmd += " --config " + config.conffile + " " + grami;
+    std::string grami = config.ControlDir()+"/job."+(*i).job_id+".grami";
+    cmd += " --config " + config.ConfigFile() + " " + grami;
     job_errors_mark_put(*i,config);
     if(!RunParallel::run(config,*i,cmd,&(i->child))) {
       if(!cancel) {
@@ -335,6 +350,11 @@ bool JobsList::state_submitting(const JobsList::iterator &i,bool &state_changed,
       }
       return false;
     }
+    ++jobs_scripts;
+    if((config.MaxScripts()!=-1) && (jobs_scripts>=config.MaxScripts())) {
+      logger.msg(Arc::WARNING,"%s: LRMS scripts limit of %u is reached - suspending submit/cancel",
+		 i->job_id,config.MaxScripts());
+    }
     return true;
   }
   // child was run - check if exited and then exit code
@@ -363,7 +383,7 @@ bool JobsList::state_submitting(const JobsList::iterator &i,bool &state_changed,
     }
     if((!simulate_success) && (Arc::Time() - i->child->RunTime()) > Arc::Period(CHILD_RUN_TIME_TOO_LONG)) {
       // In any case it is way too long. Job must fail. Otherwise it will hang forever.
-      delete i->child; i->child=NULL;
+      CleanChildProcess(i);
       if(!cancel) {
         logger.msg(Arc::ERROR,"%s: Job submission to LRMS takes too long. Failing.",i->job_id);
         JobFailStateRemember(i,JOB_STATE_SUBMITTING);
@@ -372,7 +392,7 @@ bool JobsList::state_submitting(const JobsList::iterator &i,bool &state_changed,
         return false;
       } else {
         logger.msg(Arc::ERROR,"%s: Job cancellation takes too long. Failing.",i->job_id);
-        delete i->child; i->child=NULL;
+        CleanChildProcess(i);
         return false;
       }
     }
@@ -384,7 +404,7 @@ bool JobsList::state_submitting(const JobsList::iterator &i,bool &state_changed,
       logger.msg(Arc::INFO,"%s: state SUBMIT: child exited with code %i",i->job_id,i->child->Result());
     } else {
       if((i->child->ExitTime() != Arc::Time::UNDEFINED) &&
-         ((Arc::Time() - i->child->ExitTime()) < (config.wakeup_period*2))) {
+         ((Arc::Time() - i->child->ExitTime()) < (config.WakeupPeriod()*2))) {
         // not ideal solution
         logger.msg(Arc::INFO,"%s: state CANCELING: child exited with code %i",i->job_id,i->child->Result());
       }
@@ -399,7 +419,7 @@ bool JobsList::state_submitting(const JobsList::iterator &i,bool &state_changed,
       } else {
         logger.msg(Arc::ERROR,"%s: Failed to cancel running job",i->job_id);
       }
-      delete i->child; i->child=NULL;
+      CleanChildProcess(i);
       if(!cancel) i->AddFailure("Job submission to LRMS failed");
       return false;
     }
@@ -407,7 +427,7 @@ bool JobsList::state_submitting(const JobsList::iterator &i,bool &state_changed,
     // Just pretend everything is alright
   }
   if(!cancel) {
-    delete i->child; i->child=NULL;
+    CleanChildProcess(i);
     // success code - get LRMS job id
     std::string local_id=job_desc_handler.get_local_id(i->job_id);
     if(local_id.length() == 0) {
@@ -435,13 +455,13 @@ bool JobsList::state_submitting(const JobsList::iterator &i,bool &state_changed,
          ((Arc::Time() - i->child->ExitTime()) > Arc::Period(Arc::Time::HOUR))) {
         // it takes too long
         logger.msg(Arc::ERROR,"%s: state CANCELING: timeout waiting for cancellation",i->job_id);
-        delete i->child; i->child=NULL;
+        CleanChildProcess(i);
         return false;
       }
       return true;
     } else {
       logger.msg(Arc::INFO,"%s: state CANCELING: job diagnostics collected",i->job_id);
-      delete i->child; i->child=NULL;
+      CleanChildProcess(i);
       job_diagnostics_mark_move(*i,config);
     }
   }
@@ -641,7 +661,7 @@ time_t JobsList::PrepareCleanupTime(JobsList::iterator &i,time_t& keep_finished)
 }
 
 void JobsList::UnlockDelegation(JobsList::iterator &i) {
-  ARex::DelegationStores* delegs = config.delegations;
+  ARex::DelegationStores* delegs = config.GetDelegations();
   if(delegs) (*delegs)[config.DelegationDir()].ReleaseCred(i->job_id,true,false);
 }
 
@@ -650,7 +670,7 @@ void JobsList::ActJobUndefined(JobsList::iterator &i,
                                bool& job_error,bool& state_changed) {
         // new job - read its status from status file, but first check if it is
         // under the limit of maximum jobs allowed in the system
-        if((AcceptedJobs() < config.max_jobs) || (config.max_jobs == -1)) {
+        if((AcceptedJobs() < config.MaxJobs()) || (config.MaxJobs() == -1)) {
           job_state_t new_state=job_state_read_file(i->job_id,config);
           if(new_state == JOB_STATE_UNDEFINED) { // something failed
             logger.msg(Arc::ERROR,"%s: Reading status of new job failed",i->job_id);
@@ -677,7 +697,7 @@ void JobsList::ActJobUndefined(JobsList::iterator &i,
             // prepare information for logger
             // This call is not needed here because at higher level make_file()
             // is called for every state change
-            //config.job_log->make_file(*i,config);
+            // if(config.GetJobLog()) config.GetJobLog()->make_file(*i,config);
           } else if(new_state == JOB_STATE_FINISHED) {
             once_more=true;
             job_state_write_file(*i,config,i->job_state);
@@ -722,7 +742,7 @@ void JobsList::ActJobAccepted(JobsList::iterator &i,
           return; // go to next job
         }
         // check per-DN limit on processing jobs
-        if (config.max_jobs_per_dn > 0 && jobs_dn[i->local->DN] >= config.max_jobs_per_dn) {
+        if (config.MaxPerDN() > 0 && jobs_dn[i->local->DN] >= config.MaxPerDN()) {
           JobPending(i);
           return;
         }
@@ -781,7 +801,7 @@ void JobsList::ActJobPreparing(JobsList::iterator &i,
               state_changed=false;
               JobPending(i);
             } else if(i->local->exec.size() > 0) {
-              if((config.max_jobs_running==-1) || (RunningJobs()<config.max_jobs_running)) {
+              if((config.MaxRunning()==-1) || (RunningJobs()<config.MaxRunning())) {
                 SetJobState(i, JOB_STATE_SUBMITTING, "Pre-staging finished, passing job to LRMS");
                 state_changed=true; once_more=true;
               } else {
@@ -955,7 +975,7 @@ void JobsList::ActJobFinished(JobsList::iterator &i,
           if(i->keep_deleted) {
             // here we have to get the cache per-job dirs to be deleted
             std::list<std::string> cache_per_job_dirs;
-            CacheConfig cache_config(config.cache_params);
+            CacheConfig cache_config(config.CacheParams());
             cache_config.substitute(config, i->user);
             std::vector<std::string> conf_caches = cache_config.getCacheDirs();
             // add each dir to our list
@@ -1025,7 +1045,7 @@ bool JobsList::ActJob(JobsList::iterator &i) {
         // kill running child
         if(i->child) { 
           i->child->Kill(0);
-          delete i->child; i->child=NULL;
+          CleanChildProcess(i);
         }
         // put some explanation
         i->AddFailure("User requested to cancel the job");
@@ -1133,9 +1153,9 @@ bool JobsList::ActJob(JobsList::iterator &i) {
           // Hence here we have duplicate call for ACCEPTED state.
           // TODO: maybe introducing job state prefix VALIDATING:
           // could be used to resolve this situation.
-          if(config.cont_plugins) {
+          if(config.GetContPlugins()) {
             std::list<ContinuationPlugins::result_t> results;
-            config.cont_plugins->run(*i,config,results);
+            config.GetContPlugins()->run(*i,config,results);
             std::list<ContinuationPlugins::result_t>::iterator result = results.begin();
             while(result != results.end()) {
               // analyze results
@@ -1163,13 +1183,13 @@ bool JobsList::ActJob(JobsList::iterator &i) {
             }
           }
           // Processing to be done on state changes 
-          config.job_log->make_file(*i,config);
+          if(config.GetJobLog()) config.GetJobLog()->make_file(*i,config);
           if(i->job_state == JOB_STATE_FINISHED) {
             job_clean_finished(i->job_id,config);
-            config.job_log->finish_info(*i,config);
+            if(config.GetJobLog()) config.GetJobLog()->finish_info(*i,config);
             PrepareCleanupTime(i,i->keep_finished);
           } else if(i->job_state == JOB_STATE_PREPARING) {
-            config.job_log->start_info(*i,config);
+            if(config.GetJobLog()) config.GetJobLog()->start_info(*i,config);
           }
         }
         // send mail after error and change are processed
@@ -1277,7 +1297,7 @@ bool JobsList::RestartJobs(const std::string& cdir,const std::string& odir) {
 
 // This code is run at service restart
 bool JobsList::RestartJobs(void) {
-  std::string cdir=config.control_dir;
+  std::string cdir=config.ControlDir();
   // Jobs from old version
   bool res1 = RestartJobs(cdir,cdir+"/"+subdir_rew);
   // Jobs after service restart
@@ -1311,7 +1331,7 @@ bool JobsList::ScanJobs(const std::string& cdir,std::list<JobFDesc>& ids) {
       }
     }
   } catch(Glib::FileError& e) {
-    logger.msg(Arc::ERROR,"Failed reading control directory: %s: %s",config.control_dir, e.what());
+    logger.msg(Arc::ERROR,"Failed reading control directory: %s: %s",config.ControlDir(), e.what());
     return false;
   }
 
@@ -1352,7 +1372,7 @@ bool JobsList::ScanMarks(const std::string& cdir,const std::list<std::string>& s
       }
     }
   } catch(Glib::FileError& e) {
-    logger.msg(Arc::ERROR,"Failed reading control directory: %s",config.control_dir);
+    logger.msg(Arc::ERROR,"Failed reading control directory: %s",config.ControlDir());
     return false;
   }
 
@@ -1365,8 +1385,8 @@ bool JobsList::ScanNewJobs(void) {
   Arc::JobPerfRecord perfrecord(*config.GetJobPerfLog(), "*");
   // New jobs will be accepted only if number of jobs being processed
   // does not exceed allowed. So avoid scanning if no jobs will be allowed.
-  if((AcceptedJobs() < config.max_jobs) || (config.max_jobs == -1)) {
-    std::string cdir=config.control_dir;
+  if((AcceptedJobs() < config.MaxJobs()) || (config.MaxJobs() == -1)) {
+    std::string cdir=config.ControlDir();
     std::list<JobFDesc> ids;
     // For picking up jobs after service restart
     std::string odir=cdir+"/"+subdir_rew;
@@ -1393,7 +1413,7 @@ bool JobsList::ScanNewJobs(void) {
   return true;
 }
 
-bool JobsList::ScanOldJobs(int max_scan_time,int max_scan_jobs) {
+bool JobsList::ScanOldJobs(unsigned int max_scan_time,int max_scan_jobs) {
   Arc::JobPerfRecord perfrecord(*config.GetJobPerfLog(), "*");
 
   // We are going to scan a dir with a lot of files here. So we scan it in
@@ -1404,7 +1424,7 @@ bool JobsList::ScanOldJobs(int max_scan_time,int max_scan_jobs) {
   // are normally processed in ScanNewMarks but can also happen here.
   time_t start = time(NULL);
   if(max_scan_time < 10) max_scan_time=10; // some sane number - 10s
-  std::string cdir=config.control_dir+"/"+subdir_old;
+  std::string cdir=config.ControlDir()+"/"+subdir_old;
   try {
     if(!old_dir) {
       old_dir = new Glib::Dir(cdir);
@@ -1432,13 +1452,13 @@ bool JobsList::ScanOldJobs(int max_scan_time,int max_scan_jobs) {
               JobsList::iterator i;
               AddJobNoCheck(id.id, i, uid, gid);
               ActJob(i);
-              --max_scan_jobs;
+              if(max_scan_jobs > 0) --max_scan_jobs;
             }
           }
         }
       }
-      if(((int)(time(NULL)-start)) >= max_scan_time) break;
-      if(max_scan_jobs <= 0) break;
+      if(((unsigned int)(time(NULL)-start)) >= max_scan_time) break;
+      if(max_scan_jobs == 0) break;
     }
   } catch(Glib::FileError& e) {
     logger.msg(Arc::ERROR,"Failed reading control directory: %s",cdir);
@@ -1457,7 +1477,7 @@ bool JobsList::ScanOldJobs(int max_scan_time,int max_scan_jobs) {
 bool JobsList::ScanNewMarks(void) {
   Arc::JobPerfRecord perfrecord(*config.GetJobPerfLog(), "*");
 
-  std::string cdir=config.control_dir;
+  std::string cdir=config.ControlDir();
   std::string ndir=cdir+"/"+subdir_new;
   std::list<JobFDesc> ids;
   std::list<std::string> sfx;
@@ -1501,7 +1521,7 @@ bool JobsList::ScanAllJobs(void) {
   subdirs.push_back(std::string("/")+subdir_old); // For done jobs
   for(std::list<std::string>::iterator subdir = subdirs.begin();
                                subdir != subdirs.end();++subdir) {
-    std::string cdir=config.control_dir;
+    std::string cdir=config.ControlDir();
     std::list<JobFDesc> ids;
     std::string odir=cdir+(*subdir);
     if(!ScanJobs(odir,ids)) return false;
@@ -1526,7 +1546,7 @@ bool JobsList::AddJob(const JobId& id) {
   subdirs.push_back(std::string("/")+subdir_old); // For done jobs
   for(std::list<std::string>::iterator subdir = subdirs.begin();
                                subdir != subdirs.end();++subdir) {
-    std::string cdir=config.control_dir;
+    std::string cdir=config.ControlDir();
     std::string odir=cdir+(*subdir);
     std::string fname=odir+'/'+"job."+id+".status";
     uid_t uid;
