@@ -18,7 +18,6 @@
 #include "../files/ControlFileContent.h"
 #include "../files/JobLogFile.h"
 #include "../conf/GMConfig.h"
-#include "../misc/escaped.h"
 #include "JobLog.h"
 
 namespace ARex {
@@ -49,7 +48,7 @@ bool JobLog::SetPeriod(int new_period) {
 bool JobLog::open_stream(std::ofstream &o) {
     o.open(filename.c_str(),std::ofstream::app);
     if(!o.is_open()) return false;
-    o<<(Arc::Time().str(Arc::UserTime));
+    o<<(Arc::Time().str());
     o<<" ";
     return true;
 }
@@ -115,6 +114,10 @@ bool JobLog::RunReporter(const GMConfig &config) {
   };
   if(time(NULL) < (last_run+period)) return true; // default: once per hour
   last_run=time(NULL);
+  if (logger_name.empty()) {
+    logger.msg(Arc::ERROR,": Logger name is not specified");
+    return false;
+  }
   std::string cmd = Arc::ArcLocation::GetToolsDir()+"/"+logger_name;
   cmd += " -L";  // for long format of logging
   if(ex_period) cmd += " -E " + Arc::tostring(ex_period);
